@@ -1,13 +1,14 @@
 /*
- * Copyright (C) 2012 Free Software Foundation, Inc.
+ * Copyright (C) 2012,2013 Free Software Foundation, Inc.
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
- * Author: Olga Smolenchuk
+ * Author: Nikos Mavrogiannopoulos
  *
  * This file is part of GnuTLS.
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -20,6 +21,11 @@
  *
  */
 
+/* This file implements the TLS heartbeat extension.
+ * It was originally implemented during Google SoC 2012 by by Olga Smolenchuk,
+ * and later rewritten by Nikos Mavrogiannopoulos.
+ */
+
 #include <gnutls_errors.h>
 #include <gnutls_int.h>
 #include <gnutls_dtls.h>
@@ -28,13 +34,18 @@
 #include <gnutls_extensions.h>
 #include <random.h>
 
+#ifdef ENABLE_HEARTBEAT
 /**
   * gnutls_heartbeat_enable:
   * @session: is a #gnutls_session_t structure.
   * @type: one of the GNUTLS_HB_* flags
   *
-  * This function will allow heartbeat messages to be
-  * received.
+  * If this function is called with the %GNUTLS_HB_PEER_ALLOWED_TO_SEND
+  * GnuTLS will allow heartbeat messages to be received. Moreover it also
+  * request the peer to accept heartbeat messages.
+  *
+  * The function gnutls_heartbeat_allowed() can be used to test Whether
+  * locally generated heartbeat messages can be accepted by the peer.
   *
   * Since: 3.1.2
   **/
@@ -48,22 +59,6 @@ gnutls_heartbeat_enable (gnutls_session_t session, unsigned int type)
                                 epriv);
 }
 
-/*-
- * Convenience helper:
- *
- * Returns:
- * textual policy description or NULL.
- -*/
-static inline const char *
-_gnutls_heartbeat (unsigned policy)
-{
-  if (policy & GNUTLS_HB_PEER_ALLOWED_TO_SEND)
-    return "PEER ALLOWED TO SEND";
-  else if (policy & GNUTLS_HB_PEER_NOT_ALLOWED_TO_SEND)
-    return "PEER NOT ALLOWED TO SEND";
-  return "Unknown policy";
-}
-
 /**
   * gnutls_heartbeat_allowed:
   * @session: is a #gnutls_session_t structure.
@@ -96,63 +91,44 @@ gnutls_heartbeat_allowed (gnutls_session_t session, unsigned int type)
   return 0;
 }
 
-static int
-heartbeat_allow_recv (gnutls_session_t session)
-{
-  return gnutls_heartbeat_allowed(session, GNUTLS_HB_PEER_ALLOWED_TO_SEND);
-}
-
-static int
-heartbeat_allow_send (gnutls_session_t session)
-{
-  return gnutls_heartbeat_allowed(session, GNUTLS_HB_LOCAL_ALLOWED_TO_SEND);
-}
-
 #define DEFAULT_PAYLOAD_SIZE 16
 
-/*-
- * heartbeat_send_data:
- * @session: is a #gnutls_session_t structure.
- * @data: contains the data to send.
- * @data_size: is the length of the data.
- * @start_timer: true if the heartbeat timer is to be started.
- *
- * This function has the similar semantics with gnutls_record_send() The only
- * difference is that it uses GNUTLS_HEARTBEAT content type.
- *
- * This function send either HeartBeat request or response message
- * with proper padding. It set timeout and timestamp without check - it's up to
- * caller to make sure no messages are already in-flight and handle timeout expiration.
- *
- * Returns: The number of bytes sent, or a negative error code.  The
- *   number of bytes sent might be less than @data_size.  The maximum
- *   number of bytes this function can send in a single call depends
- *   on the negotiated maximum record size.
- -*/
+/*
+ * Sends heartbeat data.
+ */
 static int
 heartbeat_send_data (gnutls_session_t session, const void *data,
                      size_t data_size, uint8_t type)
 {
-  int ret;
-  gnutls_buffer_st response;
-  uint8_t payload[DEFAULT_PAYLOAD_SIZE];
-  _gnutls_buffer_init (&response);
+  int ret, pos;
+  uint8_t * response;
 
-  ret = gnutls_rnd (GNUTLS_RND_RANDOM, payload, DEFAULT_PAYLOAD_SIZE);
+  response = gnutls_malloc(1+2+data_size+DEFAULT_PAYLOAD_SIZE);
+  if (response == NULL)
+    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
+  
+  pos = 0;
+  response[pos++] = type;
+  
+  _gnutls_write_uint16(data_size, &response[pos]);
+  pos += 2;
+  
+  memcpy(&response[pos], data, data_size);
+  pos += data_size;
+
+  ret = gnutls_rnd (GNUTLS_RND_NONCE, &response[pos], DEFAULT_PAYLOAD_SIZE);
   if (ret < 0)
-    return gnutls_assert_val(ret);
+    {
+      gnutls_assert();
+      goto cleanup;
+    }
+  pos += DEFAULT_PAYLOAD_SIZE;
 
-  BUFFER_APPEND (&response, &type, 1);
-  BUFFER_APPEND_PFX2 (&response, data, data_size);
+  ret = _gnutls_send_int (session, GNUTLS_HEARTBEAT, -1, EPOCH_WRITE_CURRENT,
+                          response, pos, MBUFFER_FLUSH);
 
-  BUFFER_APPEND (&response, payload, DEFAULT_PAYLOAD_SIZE);
-  ret = _gnutls_send_int (session, GNUTLS_HEARTBEAT, -1,
-                          EPOCH_WRITE_CURRENT, response.data,
-                          response.length, MBUFFER_FLUSH);
-
-  _gnutls_record_log ("REC[%p]: HB sent: %d\n", session, ret);
-
-  _gnutls_buffer_clear (&response);
+cleanup:
+  gnutls_free(response);
   return ret;
 }
 
@@ -185,7 +161,7 @@ gnutls_heartbeat_ping (gnutls_session_t session, size_t data_size,
   if (data_size > MAX_HEARTBEAT_LENGTH)
     return gnutls_assert_val (GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
-  if (!heartbeat_allow_send (session))
+  if (gnutls_heartbeat_allowed(session, GNUTLS_HB_LOCAL_ALLOWED_TO_SEND)==0)
     return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
   
   /* resume previous call if interrupted */
@@ -197,9 +173,6 @@ gnutls_heartbeat_ping (gnutls_session_t session, size_t data_size,
   switch(session->internals.hb_state)
     {
       case SHB_SEND1:
-        _gnutls_record_log
-         ("REC[%p]: sending HB_REQUEST with length: %zu to peer\n", session, data_size);
-         
         if (data_size > DEFAULT_PAYLOAD_SIZE)
           data_size -= DEFAULT_PAYLOAD_SIZE;
         else
@@ -254,7 +227,7 @@ retry:
               }
 
             gettime(&now);
-            diff = _dtls_timespec_sub_ms(&now, &session->internals.hb_ping_start);
+            diff = timespec_sub_ms(&now, &session->internals.hb_ping_start);
             if (diff > session->internals.hb_total_timeout_ms)
               {
                 session->internals.hb_state = SHB_SEND1;
@@ -306,43 +279,43 @@ int ret;
                              HEARTBEAT_RESPONSE);
 
   _gnutls_buffer_reset (&session->internals.hb_remote_data);
-  return ret;
+  
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
+  return 0;
 }
 
-/*-
- * Process HB message in buffer.
- * @bufel: the (suspected) HeartBeat message (TLV+padding)
- *
- * Returns:
- * processing result
- * GNUTLS_E_AGAIN if processed OK
- * GNUTLS_E_HEARTBEAT_PONG_FAILED on response send failure for request message
- * GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER on payload mismatch for response message
- -*/
+/*
+ * Processes a heartbeat message. 
+ */
 int
 _gnutls_heartbeat_handle (gnutls_session_t session, mbuffer_st * bufel)
 {
-  char pr[128];
   int ret;
+  unsigned type;
+  unsigned pos;
   uint8_t *msg = _mbuffer_get_udata_ptr (bufel);
   size_t hb_len, len = _mbuffer_get_udata_size (bufel);
 
-  if (!heartbeat_allow_recv (session))
+  if (gnutls_heartbeat_allowed(session, GNUTLS_HB_PEER_ALLOWED_TO_SEND) == 0)
     return gnutls_assert_val (GNUTLS_E_UNEXPECTED_PACKET);
 
   if (len < 4)
     return gnutls_assert_val (GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
 
-  hb_len = _gnutls_read_uint16 (msg + 1);
+  pos = 0;
+  type = msg[pos++];
+
+  hb_len = _gnutls_read_uint16 (&msg[pos]);
   if (hb_len > len - 3)
     return gnutls_assert_val (GNUTLS_E_UNEXPECTED_PACKET_LENGTH);
+    
+  pos += 2;
   
-  switch (msg[0])
+  switch (type)
     {
     case HEARTBEAT_REQUEST:
-      _gnutls_record_log
-          ("REC[%p]: received HEARTBEAT_REQUEST\n", session);
-
       _gnutls_buffer_reset(&session->internals.hb_remote_data);
 
       ret = _gnutls_buffer_resize (&session->internals.hb_remote_data, hb_len);
@@ -350,7 +323,7 @@ _gnutls_heartbeat_handle (gnutls_session_t session, mbuffer_st * bufel)
         return gnutls_assert_val(ret);
       
       if (hb_len > 0)
-        memcpy(session->internals.hb_remote_data.data, msg+3, hb_len);
+        memcpy(session->internals.hb_remote_data.data, &msg[pos], hb_len);
       session->internals.hb_remote_data.length = hb_len;
 
       return gnutls_assert_val(GNUTLS_E_HEARTBEAT_PING_RECEIVED);
@@ -360,12 +333,9 @@ _gnutls_heartbeat_handle (gnutls_session_t session, mbuffer_st * bufel)
       if (hb_len != session->internals.hb_local_data.length)
         return gnutls_assert_val (GNUTLS_E_UNEXPECTED_PACKET);
 
-      if (hb_len > 0 && memcmp (msg + 3, session->internals.hb_local_data.data,
-                  hb_len) != 0)
+      if (hb_len > 0 && 
+          memcmp (&msg[pos], session->internals.hb_local_data.data, hb_len) != 0)
         {
-          _gnutls_record_log ("REC[%p]: unexpected HB: %s - received\n", session,
-                              _gnutls_bin2hex (msg + 3, hb_len, pr,
-                                               sizeof (pr), NULL));
           if (IS_DTLS(session))
             return gnutls_assert_val( GNUTLS_E_AGAIN); /* ignore it */
           else
@@ -377,8 +347,7 @@ _gnutls_heartbeat_handle (gnutls_session_t session, mbuffer_st * bufel)
       return gnutls_assert_val(GNUTLS_E_HEARTBEAT_PONG_RECEIVED);
     default:
       _gnutls_record_log
-          ("REC[%p]: HB: received unknown type %u\n",
-           session, msg[0]);
+          ("REC[%p]: HB: received unknown type %u\n", session, type);
       return gnutls_assert_val (GNUTLS_E_UNEXPECTED_PACKET);
     }
 }
@@ -403,7 +372,7 @@ struct timespec now;
 unsigned int diff;
 
   gettime(&now);
-  diff = _dtls_timespec_sub_ms(&now, &session->internals.hb_ping_sent);
+  diff = timespec_sub_ms(&now, &session->internals.hb_ping_sent);
   if (diff >= session->internals.hb_actual_retrans_timeout_ms)
     return 0;
   else
@@ -416,15 +385,12 @@ unsigned int diff;
  * @retrans_timeout: The time at which a retransmission will occur in milliseconds
  * @total_timeout: The time at which the connection will be aborted, in milliseconds.
  *
- * This function will set the timeouts required for the DTLS handshake
+ * This function will override the timeouts for the DTLS heartbeat
  * protocol. The retransmission timeout is the time after which a
- * message from the peer is not received, the previous messages will
+ * message from the peer is not received, the previous request will
  * be retransmitted. The total timeout is the time after which the
  * handshake will be aborted with %GNUTLS_E_TIMEDOUT.
  *
- * The DTLS protocol recommends the values of 1 sec and 60 seconds
- * respectively.
- *
  * If the retransmission timeout is zero then the handshake will operate
  * in a non-blocking way, i.e., return %GNUTLS_E_AGAIN.
  *
@@ -442,7 +408,7 @@ static int
 _gnutls_heartbeat_recv_params (gnutls_session_t session,
                                const uint8_t * data, size_t _data_size)
 {
-  heartbeat_policy_t pol;
+  unsigned policy;
   extension_priv_data_t epriv;
 
   if (_gnutls_ext_get_session_data
@@ -456,24 +422,16 @@ _gnutls_heartbeat_recv_params (gnutls_session_t session,
   if (_data_size == 0)
     return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 
-  _gnutls_debug_log ("HB: received parameter %u (%zu bytes)\n",
-                     (unsigned)data[0], _data_size);
+  policy = epriv.num;
 
-  pol = epriv.num;
+  if (data[0] == 1)
+    policy |= LOCAL_ALLOWED_TO_SEND;
+  else if (data[0] == 2)
+    policy |= LOCAL_NOT_ALLOWED_TO_SEND;
+  else
+    return gnutls_assert_val (GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
 
-  switch (data[0])
-    {
-    case 1:
-      pol |= LOCAL_ALLOWED_TO_SEND;
-      break;
-    case 2:
-      pol |= LOCAL_NOT_ALLOWED_TO_SEND;
-      break;
-    default:
-      return gnutls_assert_val (GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER);
-    }
-
-  epriv.num = pol;
+  epriv.num = policy;
   _gnutls_ext_set_session_data (session, GNUTLS_EXTENSION_HEARTBEAT,
                                 epriv);
 
@@ -496,35 +454,39 @@ _gnutls_heartbeat_send_params (gnutls_session_t session,
   else /*if (epriv.num & GNUTLS_HB_PEER_NOT_ALLOWED_TO_SEND)*/
     p = 2;
 
-  _gnutls_debug_log ("HB: sending parameter %u\n", (unsigned)p);
   if (_gnutls_buffer_append_data (extdata, &p, 1) < 0)
     return gnutls_assert_val (GNUTLS_E_MEMORY_ERROR);
 
-  return 1; /* number of bytes added for sending */
+  return 1;
 }
 
 static int
-_gnutls_heartbeat_pack (extension_priv_data_t _priv, gnutls_buffer_st * ps)
+_gnutls_heartbeat_pack (extension_priv_data_t epriv, gnutls_buffer_st * ps)
 {
   int ret;
-  BUFFER_APPEND_NUM (ps, _priv.num);
-  return GNUTLS_E_SUCCESS;
+
+  BUFFER_APPEND_NUM (ps, epriv.num);
+
+  return 0;
+
 }
 
 static int
 _gnutls_heartbeat_unpack (gnutls_buffer_st * ps,
-                          extension_priv_data_t * _priv)
+                           extension_priv_data_t * _priv)
 {
-  int ret;
   extension_priv_data_t epriv;
+  int ret;
+
   BUFFER_POP_NUM (ps, epriv.num);
+
   *_priv = epriv;
+
   ret = 0;
 error:
   return ret;
 }
 
-
 extension_entry_st ext_mod_heartbeat = {
   .name = "HEARTBEAT",
   .type = GNUTLS_EXTENSION_HEARTBEAT,
@@ -536,3 +498,40 @@ extension_entry_st ext_mod_heartbeat = {
   .unpack_func = _gnutls_heartbeat_unpack,
   .deinit_func = NULL
 };
+
+#else
+void
+gnutls_heartbeat_enable (gnutls_session_t session, unsigned int type)
+{
+}
+
+int
+gnutls_heartbeat_allowed (gnutls_session_t session, unsigned int type)
+{
+  return 0;
+}
+
+int
+gnutls_heartbeat_ping (gnutls_session_t session, size_t data_size, 
+                       unsigned int max_tries, unsigned int flags)
+{
+  return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+}
+
+int
+gnutls_heartbeat_pong (gnutls_session_t session, unsigned int flags)
+{
+  return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+}
+
+unsigned int gnutls_heartbeat_get_timeout (gnutls_session_t session)
+{
+  return 0;
+}
+
+void gnutls_heartbeat_set_timeouts (gnutls_session_t session, unsigned int retrans_timeout,
+                                    unsigned int total_timeout)
+{
+  return;
+}
+#endif
