@@ -250,6 +250,9 @@ gnutls_pubkey_import_privkey (gnutls_pubkey_t key, gnutls_privkey_t pkey,
  * algorithm to use for signing with this certificate. Some certificates (i.e.
  * DSA might not be able to sign without the preferred algorithm).
  *
+ * To get the signature algorithm instead of just the hash use gnutls_pk_to_sign()
+ * with the algorithm of the certificate/key and the provided @hash.
+ *
  * Returns: the 0 if the hash algorithm is found. A negative error code is
  * returned on error.
  *
@@ -397,6 +400,13 @@ gnutls_pubkey_import_openpgp (gnutls_pubkey_t key,
   uint32_t kid32[2];
   uint32_t *k;
   uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  size_t len;
+
+  len = sizeof(key->openpgp_key_fpr);
+  ret = gnutls_openpgp_crt_get_fingerprint(crt, key->openpgp_key_fpr, &len);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+  key->openpgp_key_fpr_set = 1;
 
   ret = gnutls_openpgp_crt_get_preferred_key_id (crt, keyid);
   if (ret == GNUTLS_E_OPENPGP_PREFERRED_KEY_ERROR)
@@ -450,16 +460,19 @@ gnutls_pubkey_import_openpgp (gnutls_pubkey_t key,
 /**
  * gnutls_pubkey_get_openpgp_key_id:
  * @key: Holds the public key
- * @flags: should be 0 for now
+ * @flags: should be 0 or %GNUTLS_PUBKEY_GET_OPENPGP_FINGERPRINT
  * @output_data: will contain the key ID
  * @output_data_size: holds the size of output_data (and will be
  *   replaced by the actual size of parameters)
  * @subkey: Will be non zero if the key ID corresponds to a subkey
  *
- * This function returned the OpenPGP key ID of the corresponding key.
+ * This function returns the OpenPGP key ID of the corresponding key.
  * The key is a unique ID that depends on the public
  * key parameters. 
  *
+ * If the flag %GNUTLS_PUBKEY_GET_OPENPGP_FINGERPRINT is specified
+ * this function returns the fingerprint of the master key.
+ *
  * If the buffer provided is not long enough to hold the output, then
  * *output_data_size is updated and %GNUTLS_E_SHORT_MEMORY_BUFFER will
  * be returned.  The output is %GNUTLS_OPENPGP_KEYID_SIZE bytes long.
@@ -481,6 +494,24 @@ gnutls_pubkey_get_openpgp_key_id (gnutls_pubkey_t key, unsigned int flags,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
+  if (flags & GNUTLS_PUBKEY_GET_OPENPGP_FINGERPRINT)
+    {
+      if (*output_data_size < sizeof(key->openpgp_key_fpr))
+        {
+          *output_data_size = sizeof(key->openpgp_key_fpr);
+          return gnutls_assert_val(GNUTLS_E_SHORT_MEMORY_BUFFER);
+        }
+
+      if (key->openpgp_key_fpr_set == 0)
+        return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+      if (output_data)
+        memcpy(output_data, key->openpgp_key_fpr, sizeof(key->openpgp_key_fpr));
+      *output_data_size = sizeof(key->openpgp_key_fpr);
+      
+      return 0;
+    }
+    
   if (*output_data_size < sizeof(key->openpgp_key_id))
     {
       *output_data_size = sizeof(key->openpgp_key_id);
@@ -1243,14 +1274,20 @@ int
 gnutls_pubkey_import_url (gnutls_pubkey_t key, const char *url,
                           unsigned int flags)
 {
+  if (strncmp(url, "pkcs11:", 7) == 0)
 #ifdef ENABLE_PKCS11
-  if (strstr(url, "pkcs11:") != NULL)
     return gnutls_pubkey_import_pkcs11_url(key, url, flags);
+#else
+    return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 #endif
+
+  if (strncmp(url, "tpmkey:", 7) == 0)
 #ifdef HAVE_TROUSERS
-  if (strstr(url, "tpmkey:") != NULL)
     return gnutls_pubkey_import_tpm_url(key, url, NULL, 0);
+#else
+    return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
 #endif
+
   return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
 }
 
@@ -1336,10 +1373,6 @@ gnutls_pubkey_import_ecc_raw (gnutls_pubkey_t key,
 
   key->params.flags = curve;
 
-  ret = _gnutls_ecc_curve_fill_params(curve, &key->params);
-  if (ret < 0)
-    return gnutls_assert_val(ret);
-
   if (_gnutls_mpi_scan_nz (&key->params.params[ECC_X], x->data, x->size))
     {
       gnutls_assert ();
@@ -1497,7 +1530,7 @@ gnutls_pubkey_import_dsa_raw (gnutls_pubkey_t key,
 /**
  * gnutls_pubkey_verify_data:
  * @pubkey: Holds the public key
- * @flags: should be 0 for now
+ * @flags: Zero or one of %gnutls_pubkey_flags_t
  * @data: holds the signed data
  * @signature: contains the signature
  *
@@ -1518,6 +1551,7 @@ gnutls_pubkey_verify_data (gnutls_pubkey_t pubkey, unsigned int flags,
 			   const gnutls_datum_t * signature)
 {
   int ret;
+  gnutls_digest_algorithm_t hash;
 
   if (pubkey == NULL)
     {
@@ -1525,8 +1559,15 @@ gnutls_pubkey_verify_data (gnutls_pubkey_t pubkey, unsigned int flags,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  ret = pubkey_verify_data( pubkey->pk_algorithm, GNUTLS_DIG_UNKNOWN, data, signature,
-    &pubkey->params);
+  if (flags & GNUTLS_PUBKEY_VERIFY_FLAG_TLS1_RSA)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  ret = gnutls_pubkey_get_verify_algorithm (pubkey, signature, &hash);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
+  ret = pubkey_verify_data( pubkey->pk_algorithm, mac_to_entry(hash), 
+    data, signature, &pubkey->params);
   if (ret < 0)
     {
       gnutls_assert();
@@ -1539,7 +1580,7 @@ gnutls_pubkey_verify_data (gnutls_pubkey_t pubkey, unsigned int flags,
  * gnutls_pubkey_verify_data2:
  * @pubkey: Holds the public key
  * @algo: The signature algorithm used
- * @flags: should be 0 for now
+ * @flags: Zero or one of %gnutls_pubkey_flags_t
  * @data: holds the signed data
  * @signature: contains the signature
  *
@@ -1559,6 +1600,7 @@ gnutls_pubkey_verify_data2 (gnutls_pubkey_t pubkey,
 			    const gnutls_datum_t * signature)
 {
   int ret;
+  const mac_entry_st* me;
 
   if (pubkey == NULL)
     {
@@ -1566,7 +1608,11 @@ gnutls_pubkey_verify_data2 (gnutls_pubkey_t pubkey,
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  ret = pubkey_verify_data( pubkey->pk_algorithm, gnutls_sign_get_hash_algorithm(algo), 
+  if (flags & GNUTLS_PUBKEY_VERIFY_FLAG_TLS1_RSA)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  me = mac_to_entry(gnutls_sign_get_hash_algorithm(algo));
+  ret = pubkey_verify_data( pubkey->pk_algorithm, me,
                             data, signature, &pubkey->params);
   if (ret < 0)
     {
@@ -1580,7 +1626,7 @@ gnutls_pubkey_verify_data2 (gnutls_pubkey_t pubkey,
 /**
  * gnutls_pubkey_verify_hash:
  * @key: Holds the public key
- * @flags: should be 0 for now
+ * @flags: Zero or one of %gnutls_pubkey_flags_t
  * @hash: holds the hash digest to be verified
  * @signature: contains the signature
  *
@@ -1590,7 +1636,6 @@ gnutls_pubkey_verify_data2 (gnutls_pubkey_t pubkey,
  * Deprecated. This function cannot be easily used securely. 
  * Use gnutls_pubkey_verify_hash2() instead.
  *
- *
  * Returns: In case of a verification failure %GNUTLS_E_PK_SIG_VERIFY_FAILED 
  * is returned, and zero or positive code on success.
  *
@@ -1616,7 +1661,7 @@ int ret;
  * gnutls_pubkey_verify_hash2:
  * @key: Holds the public key
  * @algo: The signature algorithm used
- * @flags: should be 0 for now
+ * @flags: Zero or one of %gnutls_pubkey_flags_t
  * @hash: holds the hash digest to be verified
  * @signature: contains the signature
  *
@@ -1635,17 +1680,22 @@ gnutls_pubkey_verify_hash2 (gnutls_pubkey_t key,
                             const gnutls_datum_t * hash,
                             const gnutls_datum_t * signature)
 {
+  const mac_entry_st* me;
+
   if (key == NULL)
     {
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
     }
 
-  if (flags & GNUTLS_PUBKEY_VERIFY_FLAG_TLS_RSA)
-    return _gnutls_pk_verify (GNUTLS_PK_RSA, hash, signature, &key->params);
+  if (flags & GNUTLS_PUBKEY_VERIFY_FLAG_TLS1_RSA)
+    {
+      return _gnutls_pk_verify (GNUTLS_PK_RSA, hash, signature, &key->params);
+    }
   else
     {
-      return pubkey_verify_hashed_data (key->pk_algorithm, gnutls_sign_get_hash_algorithm(algo),
+      me = mac_to_entry(gnutls_sign_get_hash_algorithm(algo));
+      return pubkey_verify_hashed_data (key->pk_algorithm, me,
                                         hash, signature, &key->params);
     }
 }
@@ -1717,26 +1767,27 @@ gnutls_pubkey_get_verify_algorithm (gnutls_pubkey_t key,
  */
 int _gnutls_pubkey_compatible_with_sig(gnutls_session_t session,
                                        gnutls_pubkey_t pubkey, 
-                                       gnutls_protocol_t ver, 
+                                       const version_entry_st* ver, 
                                        gnutls_sign_algorithm_t sign)
 {
 unsigned int hash_size;
-unsigned int hash_algo;
 unsigned int sig_hash_size;
+const mac_entry_st* me;
 
   if (pubkey->pk_algorithm == GNUTLS_PK_DSA)
     {
-      hash_algo = _gnutls_dsa_q_to_hash (pubkey->pk_algorithm, &pubkey->params, &hash_size);
+      me = _gnutls_dsa_q_to_hash (pubkey->pk_algorithm, &pubkey->params, &hash_size);
 
       /* DSA keys over 1024 bits cannot be used with TLS 1.x, x<2 */
       if (!_gnutls_version_has_selectable_sighash (ver))
         {
-          if (hash_algo != GNUTLS_DIG_SHA1)
+          if (me->id != GNUTLS_MAC_SHA1)
             return gnutls_assert_val(GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL);
         }
       else if (sign != GNUTLS_SIGN_UNKNOWN)
         {
-          sig_hash_size = _gnutls_hash_get_algo_len(gnutls_sign_get_hash_algorithm(sign));
+          me = mac_to_entry(gnutls_sign_get_hash_algorithm(sign));
+          sig_hash_size = _gnutls_hash_get_algo_len(me);
           if (sig_hash_size < hash_size)
             _gnutls_audit_log(session, "The hash size used in signature (%u) is less than the expected (%u)\n", sig_hash_size, hash_size);
         }
@@ -1746,8 +1797,10 @@ unsigned int sig_hash_size;
     {
       if (_gnutls_version_has_selectable_sighash (ver) && sign != GNUTLS_SIGN_UNKNOWN)
         {
-          _gnutls_dsa_q_to_hash (pubkey->pk_algorithm, &pubkey->params, &hash_size);
-          sig_hash_size = _gnutls_hash_get_algo_len(gnutls_sign_get_hash_algorithm(sign));
+          me = _gnutls_dsa_q_to_hash (pubkey->pk_algorithm, &pubkey->params, &hash_size);
+
+          me = mac_to_entry(gnutls_sign_get_hash_algorithm(sign));
+          sig_hash_size = _gnutls_hash_get_algo_len(me);
 
           if (sig_hash_size < hash_size)
             _gnutls_audit_log(session, "The hash size used in signature (%u) is less than the expected (%u)\n", sig_hash_size, hash_size);
@@ -1783,7 +1836,7 @@ _gnutls_pubkey_get_mpis (gnutls_pubkey_t key,
  * params[1] is public key
  */
 static int
-_pkcs1_rsa_verify_sig (gnutls_digest_algorithm_t hash,
+_pkcs1_rsa_verify_sig (const mac_entry_st* me,
                        const gnutls_datum_t * text,
                        const gnutls_datum_t * prehash,
                        const gnutls_datum_t * signature, 
@@ -1795,7 +1848,7 @@ _pkcs1_rsa_verify_sig (gnutls_digest_algorithm_t hash,
   gnutls_datum_t d, di;
   digest_hd_st hd;
 
-  digest_size = _gnutls_hash_get_algo_len (hash);
+  digest_size = _gnutls_hash_get_algo_len (me);
   if (prehash)
     {
       if (prehash->data == NULL || prehash->size != digest_size)
@@ -1811,7 +1864,7 @@ _pkcs1_rsa_verify_sig (gnutls_digest_algorithm_t hash,
           return GNUTLS_E_INVALID_REQUEST;
         }
 
-      ret = _gnutls_hash_init (&hd, hash);
+      ret = _gnutls_hash_init (&hd, me);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -1829,15 +1882,13 @@ _pkcs1_rsa_verify_sig (gnutls_digest_algorithm_t hash,
 
   /* decrypted is a BER encoded data of type DigestInfo
    */
-
-  ret = encode_ber_digest_info (hash, &d, &di);
+  ret = encode_ber_digest_info (me, &d, &di);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
   ret = _gnutls_pk_verify (GNUTLS_PK_RSA, &di, signature, params);
+    _gnutls_free_datum (&di);
 
-  _gnutls_free_datum (&di);
-  
   return ret;
 }
 
@@ -1845,7 +1896,7 @@ _pkcs1_rsa_verify_sig (gnutls_digest_algorithm_t hash,
  */
 static int
 dsa_verify_hashed_data (gnutls_pk_algorithm_t pk,
-                gnutls_digest_algorithm_t algo,
+                const mac_entry_st* algo,
                 const gnutls_datum_t * hash,
                 const gnutls_datum_t * signature,
                 gnutls_pk_params_st* params)
@@ -1853,15 +1904,17 @@ dsa_verify_hashed_data (gnutls_pk_algorithm_t pk,
   gnutls_datum_t digest;
   unsigned int hash_len;
 
-  if (algo == GNUTLS_DIG_UNKNOWN)
-    algo = _gnutls_dsa_q_to_hash (pk, params, &hash_len);
-  else hash_len = _gnutls_hash_get_algo_len(algo);
-
+  if (algo == NULL)
+    algo = _gnutls_dsa_q_to_hash(pk, params, &hash_len);
+  else
+    hash_len = _gnutls_hash_get_algo_len(algo);
+  
   /* SHA1 or better allowed */
   if (!hash->data || hash->size < hash_len)
     {
       gnutls_assert();
-      _gnutls_debug_log("Hash size (%d) does not correspond to hash %s(%d) or better.\n", (int)hash->size, gnutls_mac_get_name(algo), hash_len);
+      _gnutls_debug_log("Hash size (%d) does not correspond to hash %s(%d) or better.\n", 
+        (int)hash->size, _gnutls_mac_get_name(algo), hash_len);
           
       if (hash->size != 20) /* SHA1 is allowed */
         return gnutls_assert_val(GNUTLS_E_PK_SIG_VERIFY_FAILED);
@@ -1875,7 +1928,7 @@ dsa_verify_hashed_data (gnutls_pk_algorithm_t pk,
 
 static int
 dsa_verify_data (gnutls_pk_algorithm_t pk,
-                 gnutls_digest_algorithm_t algo,
+                 const mac_entry_st* algo,
                  const gnutls_datum_t * data,
                  const gnutls_datum_t * signature,
                  gnutls_pk_params_st* params)
@@ -1885,7 +1938,7 @@ dsa_verify_data (gnutls_pk_algorithm_t pk,
   gnutls_datum_t digest;
   digest_hd_st hd;
 
-  if (algo == GNUTLS_DIG_UNKNOWN)
+  if (algo == NULL)
     algo = _gnutls_dsa_q_to_hash (pk, params, NULL);
 
   ret = _gnutls_hash_init (&hd, algo);
@@ -1906,7 +1959,7 @@ dsa_verify_data (gnutls_pk_algorithm_t pk,
  */
 int
 pubkey_verify_hashed_data (gnutls_pk_algorithm_t pk,
-                           gnutls_digest_algorithm_t hash_algo,
+                           const mac_entry_st* hash_algo,
                            const gnutls_datum_t * hash,
                            const gnutls_datum_t * signature,
                            gnutls_pk_params_st * issuer_params)
@@ -1948,7 +2001,7 @@ pubkey_verify_hashed_data (gnutls_pk_algorithm_t pk,
  */
 int
 pubkey_verify_data (gnutls_pk_algorithm_t pk,
-                    gnutls_digest_algorithm_t hash_algo,
+                    const mac_entry_st* me,
                     const gnutls_datum_t * data,
                     const gnutls_datum_t * signature,
                     gnutls_pk_params_st * issuer_params)
@@ -1959,7 +2012,7 @@ pubkey_verify_data (gnutls_pk_algorithm_t pk,
     case GNUTLS_PK_RSA:
 
       if (_pkcs1_rsa_verify_sig
-          (hash_algo, data, NULL, signature, issuer_params) != 0)
+          (me, data, NULL, signature, issuer_params) != 0)
         {
           gnutls_assert ();
           return GNUTLS_E_PK_SIG_VERIFY_FAILED;
@@ -1970,7 +2023,7 @@ pubkey_verify_data (gnutls_pk_algorithm_t pk,
 
     case GNUTLS_PK_EC:
     case GNUTLS_PK_DSA:
-      if (dsa_verify_data(pk, hash_algo, data, signature, issuer_params) != 0)
+      if (dsa_verify_data(pk, me, data, signature, issuer_params) != 0)
         {
           gnutls_assert ();
           return GNUTLS_E_PK_SIG_VERIFY_FAILED;
@@ -1985,11 +2038,12 @@ pubkey_verify_data (gnutls_pk_algorithm_t pk,
     }
 }
 
-gnutls_digest_algorithm_t
+const mac_entry_st*
 _gnutls_dsa_q_to_hash (gnutls_pk_algorithm_t algo, const gnutls_pk_params_st* params, 
                        unsigned int* hash_len)
 {
   int bits = 0;
+  int ret;
   
   if (algo == GNUTLS_PK_DSA)
     bits = _gnutls_mpi_get_nbits (params->params[1]);
@@ -1999,33 +2053,35 @@ _gnutls_dsa_q_to_hash (gnutls_pk_algorithm_t algo, const gnutls_pk_params_st* pa
   if (bits <= 160)
     {
       if (hash_len) *hash_len = 20;
-      return GNUTLS_DIG_SHA1;
+      ret = GNUTLS_DIG_SHA1;
     }
   else if (bits <= 192)
     {
       if (hash_len) *hash_len = 24;
-      return GNUTLS_DIG_SHA256;
+      ret = GNUTLS_DIG_SHA256;
     }
   else if (bits <= 224)
     {
       if (hash_len) *hash_len = 28;
-      return GNUTLS_DIG_SHA256;
+      ret = GNUTLS_DIG_SHA256;
     }
   else if (bits <= 256)
     {
       if (hash_len) *hash_len = 32;
-      return GNUTLS_DIG_SHA256;
+      ret = GNUTLS_DIG_SHA256;
     }
   else if (bits <= 384)
     {
       if (hash_len) *hash_len = 48;
-      return GNUTLS_DIG_SHA384;
+      ret = GNUTLS_DIG_SHA384;
     }
   else
     {
       if (hash_len) *hash_len = 64;
-      return GNUTLS_DIG_SHA512;
+      ret = GNUTLS_DIG_SHA512;
     }
+    
+  return mac_to_entry(ret);
 }
 
 /**
