@@ -457,6 +457,7 @@ print_info (gnutls_session_t session, int verbose, int print_cert)
     unsigned char session_id[33];
     size_t session_id_size = sizeof (session_id);
     gnutls_srtp_profile_t srtp_profile;
+    gnutls_datum_t p;
     char *desc;
     int rc;
 
@@ -547,6 +548,18 @@ print_info (gnutls_session_t session, int verbose, int print_cert)
     tmp = SU (gnutls_kx_get_name (kx));
     printf ("- Key Exchange: %s\n", tmp);
 
+    if (gnutls_sign_algorithm_get(session) != GNUTLS_SIGN_UNKNOWN)
+      {
+        tmp = SU (gnutls_sign_get_name (gnutls_sign_algorithm_get (session)));
+        printf ("- Server Signature: %s\n", tmp);
+      }
+
+    if (gnutls_sign_algorithm_get_client(session) != GNUTLS_SIGN_UNKNOWN)
+      {
+        tmp = SU (gnutls_sign_get_name (gnutls_sign_algorithm_get_client (session)));
+        printf ("- Client Signature: %s\n", tmp);
+      }
+
     tmp = SU (gnutls_cipher_get_name (gnutls_cipher_get (session)));
     printf ("- Cipher: %s\n", tmp);
 
@@ -564,6 +577,10 @@ print_info (gnutls_session_t session, int verbose, int print_cert)
       printf ("- SRTP profile: %s\n", gnutls_srtp_get_profile_name (srtp_profile));
 #endif
 
+    rc = gnutls_alpn_get_selected_protocol (session, &p);
+    if (rc == 0)
+      printf ("- Application protocol: %.*s\n", p.size, p.data);
+
     if (verbose)
       {
           gnutls_datum_t cb;
@@ -847,6 +864,20 @@ print_list (const char *priorities, int verbose)
           }
     }
 
+    {
+        const gnutls_digest_algorithm_t *p = gnutls_digest_list ();
+
+        printf ("Digests: ");
+        for (; *p; p++)
+          {
+              printf ("%s", gnutls_digest_get_name (*p));
+              if (*(p + 1))
+                  printf (", ");
+              else
+                  printf ("\n");
+          }
+    }
+
     {
         const gnutls_kx_algorithm_t *p = gnutls_kx_list ();
 
@@ -960,7 +991,8 @@ pin_callback (void *user, int attempt, const char *token_url,
 {
   const char *password;
   const char * desc;
-  int len, cache = MAX_CACHE_TRIES;
+  int cache = MAX_CACHE_TRIES;
+  unsigned len;
 /* allow caching of PIN */
   static char *cached_url = NULL;
   static char cached_pin[32] = "";
@@ -1014,12 +1046,19 @@ pin_callback (void *user, int attempt, const char *token_url,
       exit (1);
     }
 
-  len = MIN (pin_max, strlen (password));
+  len = MIN (pin_max-1, strlen (password));
   memcpy (pin, password, len);
   pin[len] = 0;
 
   /* cache */
-  strcpy (cached_pin, pin);
+  if (len < sizeof(cached_pin))
+    {
+      memcpy (cached_pin, pin, len);
+      cached_pin[len] = 0;
+    }
+  else
+    cached_pin[0] = 0;
+
   free (cached_url);
   if (token_url)
     cached_url = strdup (token_url);
