@@ -186,7 +186,7 @@ client (int sds[])
        */
       gnutls_credentials_set (session, GNUTLS_CRD_CERTIFICATE, xcred);
 
-      gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) sd);
+      gnutls_transport_set_int (session, sd);
 
       /* Perform the TLS handshake
        */
@@ -532,7 +532,7 @@ server (int sds[])
 
       session = initialize_tls_session ();
 
-      gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) sd);
+      gnutls_transport_set_int (session, sd);
       ret = gnutls_handshake (session);
       if (ret < 0)
         {
